# frozen_string_literal: true

module Bulkrax
  class CrcFolderComplexSessionEntry < CrcFolderEntry
    include ComplexHelper
    def build
      begin
        raise ComplexSubjectCreatedError unless subject_created?

        complex_session = ComplexSession.new(session_title: validate_and_fix_title, session_date_recorded: self.raw_metadata['record_date'])
        complex_session.source_identifier = identifier
        complex_session.parent_source_identifier = self.raw_metadata['parents']

        complex_session.save!
      rescue RSolr::Error::Http, ComplexSubjectCreatedError => e
        raise e
      rescue StandardError => e
        status_info(e)
      else
        status_info
      ensure
        self.save!
      end
    end

    def factory_class
      ComplexSession
    end

    def subject_created?
      ComplexSubject.find_by(source_identifier: self.raw_metadata['parents'])
    end

    def validate_and_fix_title
      parent = ::ComplexSubject.find_by(source_identifier: self.raw_metadata['parents'])

      siblings = parent.complex_sessions
      session_title = sanitize_title(self.raw_metadata['title'])
     
      session_title = session_title.rjust(3, '0')  if session_title.length < 3 

      suffix = 1
      new_title = session_title
  
      if siblings.present?
        while siblings.where(session_title: new_title).present?
          new_title = "#{session_title}_#{suffix}"
          suffix += 1
        end
      end

      new_title
    end
  end
end
