# frozen_string_literal: true

module Bulkrax
  class CrcFolderComplexSubjectEntry < CrcFolderEntry
    include ComplexHelper
    def build
      begin
        raise CrcDatasetCreatedError unless crc_dataset_created?

        complex_subject = ComplexSubject.new(self.raw_metadata.slice('subject_species', 'subject_type', 'subject_sex', 'subject_age'))
        complex_subject.subject_title = validate_and_fix_title
        complex_subject.source_identifier = identifier
        complex_subject.parent_source_identifier = self.raw_metadata['parents']

        complex_subject.save!
      rescue RSolr::Error::Http, CrcDatasetCreatedError => e
        raise e
      rescue StandardError => e
        status_info(e)
      else
        status_info
      ensure
        self.save!
      end
    end

    def factory_class
      ComplexSubject
    end

    def crc_dataset_created?
      ::CrcDataset.where(source: self.raw_metadata['parents']).present?
    end

    def validate_and_fix_title
      parent = ::CrcDataset.where(source: self.raw_metadata['parents']).first
      siblings = parent.complex_subjects
      subject_title = sanitize_title(self.raw_metadata['title'])
     
      subject_title = subject_title.rjust(3, '0')  if subject_title.length < 3 

      suffix = 1
      new_title = subject_title

      if siblings.present?
        while siblings.where(subject_title: new_title).present?
          new_title = "#{subject_title}_#{suffix}"
          suffix += 1
        end
      end

      new_title
    end
  end
end
