module ExternalServices
  extend ActiveSupport::Concern
  include HyraxHelper
  included do

    def register_ark
      return true if ENV.fetch('REGISTER_ARK', 'true') == 'false'
      ark = ArkService.new
      ark.authenticate

      creators = self.complex_person.map{|person| "#{person.last_name.first},#{person.first_name.first}" }.compact

      metadata = {
        'creator': creators.join(';'),
        'title': self.title.first,
        'date_created': self.create_date
      }
      # url = "#{ENV['APPLICATION_URL']}/concern/crc_datasets/#{self.id}"
      url = Rails.application.routes.url_helpers.polymorphic_url(self, host: ENV['APPLICATION_URL'])
      resp = ark.create_ark(self.id, url, metadata)
      if resp and resp[0]
        # Save ark to metadata
        complex_identifier = ComplexIdentifier.new(RDF::Node.new, ActiveTriples::Resource.new)
        complex_identifier.identifier = resp[2]
        complex_identifier.scheme = 'ARK'
        self.update(complex_identifier: self.complex_identifier + [complex_identifier])
        self.save
        # mark ark private
        ark.make_ark_private(self.id)
      end
    end

    def total_file_count_and_size
      s3 = S3StorageService.new
      s3.init_client
      bucket_name = s3.sanitise_name(self.id)
      return 0 unless s3.bucket_exists?(bucket_name)
      _list_of_objects, size = s3.list_all_objects(bucket_name)
      size
      [_list_of_objects.count, size]
    end

    def save_work_meta_json_file_to_s3
      s3 = S3StorageService.new
      s3.init_client
      bucket_name = s3.sanitise_name(id)
      self_object = self.class.find(self.id)
      s3.create_bucket(bucket_name) unless s3.bucket_exists?(bucket_name)
      s3.add_content(bucket_name, 'system_metadata.json', self_object.to_json)
      s3.add_content(bucket_name, "metadata.json", filter_metadata_json_file(self_object))
    end

  end
end